@@ -21,15 +21,21 @@
 #include "geometry.h"
 #include "color.h"
 #include "diagram_tree_window.h"
+#include "diagramdata.h"
+
+#define DEFAULT_GRID_COLOR { 0.85, .90, .90 }
+#define DEFAULT_PAGEBREAK_COLOR { 0.0, 0.0, 0.6 }
 
 struct DiaPreferences {
   struct {
     int visible;
     int snap;
+    gboolean dynamic;
     real x;
     real y;
-    Color colour;
-    int solid;
+    int major_lines;
+    int hex;
+    real w;
   } grid;
   
   struct {
@@ -39,31 +45,25 @@ struct DiaPreferences {
     int use_menu_bar;
   } new_view;
 
-  struct {
-    gchar *papertype;
-    gfloat tmargin, bmargin, lmargin, rmargin;
-    gboolean is_portrait;
-    gfloat scaling;
-    gboolean fitto;
-    gint fitwidth, fitheight;
-  } new_diagram;
-  
+  NewDiagramData new_diagram;
+
   int show_cx_pts;
   int reset_tools_after_create;
-  int compress_save;
   int undo_depth;
   int reverse_rubberbanding_intersects;
   guint recent_documents_list_size;
   
   struct {
     int visible;
-    Color colour;
     int solid;
   } pagebreak;
 
   int render_bounding_boxes;
-  int pretty_formated_xml;
   int prefer_psprint;
+    
+  int toolbox_on_top;
+  int recent_sheet;
+
   DiagramTreeConfig dia_tree;
 };
 
@@ -72,7 +72,7 @@ extern struct DiaPreferences prefs;
 void prefs_show(void);
 void prefs_set_defaults(void);
 void prefs_save(void);
-void prefs_load(void);
+void prefs_init(void);
 
 #endif /* DIA_IMAGE_H */
 
