@@ -23,12 +23,12 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <assert.h>
 #include <math.h>
 #include <string.h> /* memcpy() */
 
 #include "beziershape.h"
 #include "message.h"
+#include "diarenderer.h"
 
 #define HANDLE_BEZMAJOR  (HANDLE_CUSTOM1)
 #define HANDLE_LEFTCTRL  (HANDLE_CUSTOM2)
@@ -82,6 +82,8 @@ beziershape_create_corner_change(BezierShape *bezier, Handle *handle,
 			BezCornerType old_corner_type,
 			BezCornerType new_corner_type);
 
+static void new_handles_and_connections(BezierShape *bezier, int num_points);
+
 static void setup_handle(Handle *handle, int handle_id)
 {
   handle->id = handle_id;
@@ -107,9 +109,10 @@ static int get_handle_nr(BezierShape *bezier, Handle *handle)
 #define get_comp_nr(hnum) ((int)(hnum)/3+1)
 #define get_major_nr(hnum) (((int)(hnum)+2)/3)
 
-void
+ObjectChange *
 beziershape_move_handle(BezierShape *bezier, Handle *handle,
-			Point *to, HandleMoveReason reason)
+			Point *to, ConnectionPoint *cp,
+			HandleMoveReason reason, ModifierKeys modifiers)
 {
   int handle_nr, comp_nr, next_nr, prev_nr;
   Point delta, pt;
@@ -209,9 +212,10 @@ beziershape_move_handle(BezierShape *bezier, Handle *handle,
     message_error("Internal error in beziershape_move_handle.");
     break;
   }
+  return NULL;
 }
 
-void
+ObjectChange*
 beziershape_move(BezierShape *bezier, Point *to)
 {
   Point p;
@@ -226,6 +230,8 @@ beziershape_move(BezierShape *bezier, Point *to)
     point_add(&bezier->points[i].p2, &p);
     point_add(&bezier->points[i].p3, &p);
   }
+
+  return NULL;
 }
 
 int
@@ -309,12 +315,12 @@ add_handles(BezierShape *bezier, int pos, BezPoint *point,
 	    ConnectionPoint *cp1, ConnectionPoint *cp2)
 {
   int i, next;
-  Object *obj;
+  DiaObject *obj;
 
   g_assert(pos >= 1);
   g_assert(pos <= bezier->numpoints);
 
-  obj = (Object *)bezier;
+  obj = (DiaObject *)bezier;
   bezier->numpoints++;
   next = pos + 1;
   if (pos == bezier->numpoints - 1)
@@ -334,18 +340,18 @@ add_handles(BezierShape *bezier, int pos, BezPoint *point,
   if (pos == bezier->numpoints - 1)
     bezier->points[0].p1 = bezier->points[0].p3 = bezier->points[pos].p3;
   bezier->corner_types[pos] = corner_type;
-  object_add_handle_at((Object*)bezier, handle1, 3*pos-3);
-  object_add_handle_at((Object*)bezier, handle2, 3*pos-2);
-  object_add_handle_at((Object*)bezier, handle3, 3*pos-1);
-  object_add_connectionpoint_at((Object *)bezier, cp1, 2*pos-1);
-  object_add_connectionpoint_at((Object *)bezier, cp2, 2*pos);
+  object_add_handle_at((DiaObject*)bezier, handle1, 3*pos-3);
+  object_add_handle_at((DiaObject*)bezier, handle2, 3*pos-2);
+  object_add_handle_at((DiaObject*)bezier, handle3, 3*pos-1);
+  object_add_connectionpoint_at((DiaObject *)bezier, cp1, 2*pos-1);
+  object_add_connectionpoint_at((DiaObject *)bezier, cp2, 2*pos);
 }
 
 static void
 remove_handles(BezierShape *bezier, int pos)
 {
   int i;
-  Object *obj;
+  DiaObject *obj;
   Handle *old_handle1, *old_handle2, *old_handle3;
   ConnectionPoint *old_cp1, *old_cp2;
   Point tmppoint;
@@ -354,7 +360,7 @@ remove_handles(BezierShape *bezier, int pos)
   g_assert(pos > 0);
   g_assert(pos < bezier->numpoints);
 
-  obj = (Object *)bezier;
+  obj = (DiaObject *)bezier;
 
   /* delete the points */
   bezier->numpoints--;
@@ -467,9 +473,9 @@ beziershape_remove_segment(BezierShape *bezier, int pos)
   old_cp1 = bezier->object.connections[2*pos-2];
   old_cp2 = bezier->object.connections[2*pos-1];
   
-  object_unconnect((Object *)bezier, old_handle1);
-  object_unconnect((Object *)bezier, old_handle2);
-  object_unconnect((Object *)bezier, old_handle3);
+  object_unconnect((DiaObject *)bezier, old_handle1);
+  object_unconnect((DiaObject *)bezier, old_handle2);
+  object_unconnect((DiaObject *)bezier, old_handle3);
 
   remove_handles(bezier, pos);
 
@@ -607,6 +613,33 @@ beziershape_update_data(BezierShape *bezier)
   int i;
   Point last;
   
+  DiaObject *obj = &bezier->object;
+  
+  /* handle the case of whole points array update (via set_prop) */
+  if (3*(bezier->numpoints-1) != obj->num_handles ||
+      2*(bezier->numpoints-1) != obj->num_connections) {
+    object_unconnect_all(obj); /* too drastic ? */
+
+    /* delete the old ones */
+    for (i = 0; i < obj->num_handles; i++)
+      g_free(obj->handles[i]);
+    g_free(obj->handles);
+    for (i = 0; i < obj->num_connections; i++);
+      g_free(obj->connections[i]);
+    g_free(obj->connections);
+
+    obj->num_handles = 3*(bezier->numpoints-1);
+    obj->handles = g_new(Handle*, obj->num_handles);
+    obj->num_connections = 2*(bezier->numpoints-1);
+    obj->connections = g_new(ConnectionPoint *, obj->num_connections);
+
+    new_handles_and_connections(bezier, bezier->numpoints);
+
+    bezier->corner_types = g_realloc(bezier->corner_types, bezier->numpoints*sizeof(BezCornerType));
+    for (i = 0; i < bezier->numpoints; i++)
+      bezier->corner_types[i] = BEZ_CORNER_SYMMETRIC;
+  }
+
   /* Update handles: */
   bezier->points[0].p3 = bezier->points[0].p1;
   for (i = 1; i < bezier->numpoints; i++) {
@@ -618,13 +651,27 @@ beziershape_update_data(BezierShape *bezier)
   /* Update connection points: */
   last = bezier->points[0].p1;
   for (i = 1; i < bezier->numpoints; i++) {
+    Point slopepoint1, slopepoint2;
+    slopepoint1 = bezier->points[i].p1;
+    point_sub(&slopepoint1, &last);
+    point_scale(&slopepoint1, .5);
+    point_add(&slopepoint1, &last);
+    slopepoint2 = bezier->points[i].p2;
+    point_sub(&slopepoint2, &bezier->points[i].p3);
+    point_scale(&slopepoint2, .5);
+    point_add(&slopepoint2, &bezier->points[i].p3);
+
     bezier->object.connections[2*i-2]->pos = last;
+    bezier->object.connections[2*i-2]->directions =
+      find_slope_directions(last, bezier->points[i].p1);
     bezier->object.connections[2*i-1]->pos.x =
       (last.x + 3*bezier->points[i].p1.x + 3*bezier->points[i].p2.x +
        bezier->points[i].p3.x)/8;
     bezier->object.connections[2*i-1]->pos.y =
       (last.y + 3*bezier->points[i].p1.y + 3*bezier->points[i].p2.y +
        bezier->points[i].p3.y)/8;
+    bezier->object.connections[2*i-1]->directions = 
+      find_slope_directions(slopepoint1, slopepoint2);
     last = bezier->points[i].p3;
   }
 }
@@ -635,7 +682,7 @@ beziershape_update_boundingbox(BezierShape *bezier)
   ElementBBExtras *extra;
   PolyBBExtras pextra;
 
-  assert(bezier != NULL);
+  g_assert(bezier != NULL);
 
   extra = &bezier->extra_spacing;
   pextra.start_trans = pextra.end_trans = 
@@ -649,125 +696,134 @@ beziershape_update_boundingbox(BezierShape *bezier)
 }
 
 void
-beziershape_simple_draw(BezierShape *bezier, Renderer *renderer, real width)
+beziershape_simple_draw(BezierShape *bezier, DiaRenderer *renderer, real width)
 {
   BezPoint *points;
   
-  assert(bezier != NULL);
-  assert(renderer != NULL);
+  g_assert(bezier != NULL);
+  g_assert(renderer != NULL);
 
   points = &bezier->points[0];
   
-  renderer->ops->set_linewidth(renderer, width);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_ROUND);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, width);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_ROUND);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
 
-  renderer->ops->fill_bezier(renderer, points, bezier->numpoints,&color_white);
-  renderer->ops->draw_bezier(renderer, points, bezier->numpoints,&color_black);
+  DIA_RENDERER_GET_CLASS(renderer)->fill_bezier(renderer, points, bezier->numpoints,&color_white);
+  DIA_RENDERER_GET_CLASS(renderer)->draw_bezier(renderer, points, bezier->numpoints,&color_black);
 }
 
 void
-beziershape_draw_control_lines(BezierShape *bez, Renderer *renderer)
+beziershape_draw_control_lines(BezierShape *bez, DiaRenderer *renderer)
 {
   Color line_colour = {0.0, 0.0, 0.6};
   Point startpoint;
   int i;
   
   /* setup renderer ... */
-  renderer->ops->set_linewidth(renderer, 0);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_DOTTED);
-  renderer->ops->set_dashlength(renderer, 1);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, 0);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_DOTTED);
+  DIA_RENDERER_GET_CLASS(renderer)->set_dashlength(renderer, 1);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
 
   startpoint = bez->points[0].p1;
   for (i = 1; i < bez->numpoints; i++) {
-    renderer->ops->draw_line(renderer, &startpoint, &bez->points[i].p1,
+    DIA_RENDERER_GET_CLASS(renderer)->draw_line(renderer, &startpoint, &bez->points[i].p1,
                              &line_colour);
-    renderer->ops->draw_line(renderer, &bez->points[i].p2, &bez->points[i].p3,
+    DIA_RENDERER_GET_CLASS(renderer)->draw_line(renderer, &bez->points[i].p2, &bez->points[i].p3,
                              &line_colour);
     startpoint = bez->points[i].p3;
   }
 }
 
-void
-beziershape_init(BezierShape *bezier)
+static void
+new_handles_and_connections(BezierShape *bezier, int num_points)
 {
-  Object *obj;
+  DiaObject *obj;
+  int i;
 
   obj = &bezier->object;
 
-  object_init(obj, 6, 4);
+  for (i = 0; i < num_points-1; i++) {
+    obj->handles[3*i] = g_new(Handle, 1);
+    obj->handles[3*i+1] = g_new(Handle, 1);
+    obj->handles[3*i+2] = g_new(Handle, 1);
   
-  bezier->numpoints = 3;
+    obj->handles[3*i]->connect_type = HANDLE_NONCONNECTABLE;
+    obj->handles[3*i]->connected_to = NULL;
+    obj->handles[3*i]->type = HANDLE_MINOR_CONTROL;
+    obj->handles[3*i]->id = HANDLE_RIGHTCTRL;
 
-  bezier->points = g_new(BezPoint, 3);
+    obj->handles[3*i+1]->connect_type = HANDLE_NONCONNECTABLE;
+    obj->handles[3*i+1]->connected_to = NULL;
+    obj->handles[3*i+1]->type = HANDLE_MINOR_CONTROL;
+    obj->handles[3*i+1]->id = HANDLE_LEFTCTRL;
+  
+    obj->handles[3*i+2]->connect_type = HANDLE_NONCONNECTABLE;
+    obj->handles[3*i+2]->connected_to = NULL;
+    obj->handles[3*i+2]->type = HANDLE_MAJOR_CONTROL;
+    obj->handles[3*i+2]->id = HANDLE_BEZMAJOR;
+
+    obj->connections[2*i] = g_new0(ConnectionPoint, 1);
+    obj->connections[2*i+1] = g_new0(ConnectionPoint, 1);
+    obj->connections[2*i]->object = obj;
+    obj->connections[2*i+1]->object = obj;
+  }
+}
+
+void
+beziershape_init(BezierShape *bezier, int num_points)
+{
+  DiaObject *obj;
+  int i;
+
+  obj = &bezier->object;
+
+  object_init(obj, 3*(num_points-1), 2*(num_points-1));
+  
+  bezier->numpoints = num_points;
+
+  bezier->points = g_new(BezPoint, num_points);
   bezier->points[0].type = BEZ_MOVE_TO;
-  bezier->points[1].type = BEZ_CURVE_TO;
-  bezier->points[2].type = BEZ_CURVE_TO;
+  bezier->corner_types = g_new(BezCornerType, num_points);
+  for (i = 1; i < num_points; i++) {
+    bezier->points[i].type = BEZ_CURVE_TO;
+    bezier->corner_types[i] = BEZ_CORNER_SYMMETRIC;
+  }
 
-  bezier->corner_types = g_new(BezCornerType, 3);
-  bezier->corner_types[0] = BEZ_CORNER_SYMMETRIC;
-  bezier->corner_types[1] = BEZ_CORNER_SYMMETRIC;
-  bezier->corner_types[2] = BEZ_CORNER_SYMMETRIC;
-
-  bezier->object.handles[0] = g_new(Handle, 1);
-  bezier->object.handles[1] = g_new(Handle, 1);
-  bezier->object.handles[2] = g_new(Handle, 1);
-  bezier->object.handles[3] = g_new(Handle, 1);
-  bezier->object.handles[4] = g_new(Handle, 1);
-  bezier->object.handles[5] = g_new(Handle, 1);
-
-  obj->handles[0]->connect_type = HANDLE_NONCONNECTABLE;
-  obj->handles[0]->connected_to = NULL;
-  obj->handles[0]->type = HANDLE_MINOR_CONTROL;
-  obj->handles[0]->id = HANDLE_RIGHTCTRL;
-  
-  obj->handles[1]->connect_type = HANDLE_NONCONNECTABLE;
-  obj->handles[1]->connected_to = NULL;
-  obj->handles[1]->type = HANDLE_MINOR_CONTROL;
-  obj->handles[1]->id = HANDLE_LEFTCTRL;
-
-  obj->handles[2]->connect_type = HANDLE_NONCONNECTABLE;
-  obj->handles[2]->connected_to = NULL;
-  obj->handles[2]->type = HANDLE_MAJOR_CONTROL;
-  obj->handles[2]->id = HANDLE_BEZMAJOR;
-
-  obj->handles[3]->connect_type = HANDLE_NONCONNECTABLE;
-  obj->handles[3]->connected_to = NULL;
-  obj->handles[3]->type = HANDLE_MINOR_CONTROL;
-  obj->handles[3]->id = HANDLE_RIGHTCTRL;
-  
-  obj->handles[4]->connect_type = HANDLE_NONCONNECTABLE;
-  obj->handles[4]->connected_to = NULL;
-  obj->handles[4]->type = HANDLE_MINOR_CONTROL;
-  obj->handles[4]->id = HANDLE_LEFTCTRL;
-
-  obj->handles[5]->connect_type = HANDLE_NONCONNECTABLE;
-  obj->handles[5]->connected_to = NULL;
-  obj->handles[5]->type = HANDLE_MAJOR_CONTROL;
-  obj->handles[5]->id = HANDLE_BEZMAJOR;
-
-  obj->connections[0] = g_new0(ConnectionPoint, 1);
-  obj->connections[1] = g_new0(ConnectionPoint, 1);
-  obj->connections[2] = g_new0(ConnectionPoint, 1);
-  obj->connections[3] = g_new0(ConnectionPoint, 1);
-  obj->connections[0]->object = obj;
-  obj->connections[1]->object = obj;
-  obj->connections[2]->object = obj;
-  obj->connections[3]->object = obj;
+  new_handles_and_connections(bezier, num_points);
 
   /* The points are not assigned at this point, so don't try to use
      them */
   /*  beziershape_update_data(bezier);*/
 }
 
+
+/** This function does *not* set up handles */
+void
+beziershape_set_points(BezierShape *bez, int num_points, BezPoint *points)
+{
+  int i;
+
+  bez->numpoints = num_points;
+
+  if (bez->points)
+    g_free(bez->points);
+
+  bez->points = g_malloc((bez->numpoints)*sizeof(BezPoint));
+
+  for (i=0;i<bez->numpoints;i++) {
+    bez->points[i] = points[i];
+  }
+}
+
 void
 beziershape_copy(BezierShape *from, BezierShape *to)
 {
   int i;
-  Object *toobj, *fromobj;
+  DiaObject *toobj, *fromobj;
 
   toobj = &to->object;
   fromobj = &from->object;
@@ -859,7 +915,7 @@ beziershape_load(BezierShape *bezier, ObjectNode obj_node)
   AttributeNode attr;
   DataNode data;
   
-  Object *obj = &bezier->object;
+  DiaObject *obj = &bezier->object;
 
   object_load(obj, obj_node);
 
@@ -943,7 +999,7 @@ beziershape_point_change_free(struct PointChange *change)
 }
 
 static void
-beziershape_point_change_apply(struct PointChange *change, Object *obj)
+beziershape_point_change_apply(struct PointChange *change, DiaObject *obj)
 {
   change->applied = 1;
   switch (change->type) {
@@ -963,7 +1019,7 @@ beziershape_point_change_apply(struct PointChange *change, Object *obj)
 }
 
 static void
-beziershape_point_change_revert(struct PointChange *change, Object *obj)
+beziershape_point_change_revert(struct PointChange *change, DiaObject *obj)
 {
   switch (change->type) {
   case TYPE_ADD_POINT:
@@ -1013,7 +1069,7 @@ beziershape_create_point_change(BezierShape *bezier, enum change_type type,
 }
 
 static void
-beziershape_corner_change_apply(struct CornerChange *change, Object *obj)
+beziershape_corner_change_apply(struct CornerChange *change, DiaObject *obj)
 {
   BezierShape *bez = (BezierShape *)obj;
   int handle_nr = get_handle_nr(bez, change->handle);
@@ -1031,7 +1087,7 @@ beziershape_corner_change_apply(struct CornerChange *change, Object *obj)
 }
 
 static void
-beziershape_corner_change_revert(struct CornerChange *change, Object *obj)
+beziershape_corner_change_revert(struct CornerChange *change, DiaObject *obj)
 {
   BezierShape *bez = (BezierShape *)obj;
   int handle_nr = get_handle_nr(bez, change->handle);
