@@ -27,11 +27,12 @@
 #include "intl.h"
 #include "object.h"
 #include "connection.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "handle.h"
 #include "arrows.h"
 #include "properties.h"
 #include "stereotype.h"
+#include "attributes.h"
 
 #include "pixmaps/constraint.xpm"
 
@@ -42,10 +43,13 @@ struct _Constraint {
 
   Handle text_handle;
 
-  utfchar *text;
-  utfchar *brtext;
+  char *text;
+  char *brtext;
   Point text_pos;
   real text_width;
+
+  Color text_color;
+  Color line_color;
 };
 
   
@@ -60,13 +64,14 @@ struct _Constraint {
 
 static DiaFont *constraint_font = NULL;
 
-static void constraint_move_handle(Constraint *constraint, Handle *handle,
-				   Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void constraint_move(Constraint *constraint, Point *to);
+static ObjectChange* constraint_move_handle(Constraint *constraint, Handle *handle,
+					    Point *to, ConnectionPoint *cp,
+					    HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* constraint_move(Constraint *constraint, Point *to);
 static void constraint_select(Constraint *constraint, Point *clicked_point,
-			      Renderer *interactive_renderer);
-static void constraint_draw(Constraint *constraint, Renderer *renderer);
-static Object *constraint_create(Point *startpoint,
+			      DiaRenderer *interactive_renderer);
+static void constraint_draw(Constraint *constraint, DiaRenderer *renderer);
+static DiaObject *constraint_create(Point *startpoint,
 				 void *user_data,
 				 Handle **handle1,
 				 Handle **handle2);
@@ -78,7 +83,7 @@ static PropDescription *constraint_describe_props(Constraint *constraint);
 static void constraint_get_props(Constraint * constraint, GPtrArray *props);
 static void constraint_set_props(Constraint * constraint, GPtrArray *props);
 
-static Object *constraint_load(ObjectNode obj_node, int version,
+static DiaObject *constraint_load(ObjectNode obj_node, int version,
 			       const char *filename);
 
 static ObjectTypeOps constraint_type_ops =
@@ -90,7 +95,7 @@ static ObjectTypeOps constraint_type_ops =
   (ApplyDefaultsFunc) NULL
 };
 
-ObjectType constraint_type =
+DiaObjectType constraint_type =
 {
   "UML - Constraint",        /* name */
   0,                         /* version */
@@ -119,12 +124,17 @@ static PropDescription constraint_props[] = {
   { "constraint", PROP_TYPE_STRING, PROP_FLAG_VISIBLE,
     N_("Constraint:"), NULL, NULL },
   { "text_pos", PROP_TYPE_POINT, 0, NULL, NULL, NULL},
+  PROP_STD_TEXT_COLOUR_OPTIONAL,
+  PROP_STD_LINE_COLOUR_OPTIONAL, 
   PROP_DESC_END
 };
 
 static PropDescription *
 constraint_describe_props(Constraint *constraint)
 {
+  if (constraint_props[0].quark == 0) {
+    prop_desc_list_calculate_quarks(constraint_props);
+  }
   return constraint_props;
 }
 
@@ -132,6 +142,8 @@ static PropOffset constraint_offsets[] = {
   CONNECTION_COMMON_PROPERTIES_OFFSETS,
   { "constraint", PROP_TYPE_STRING, offsetof(Constraint, text) },
   { "text_pos", PROP_TYPE_POINT, offsetof(Constraint, text_pos) },
+  { "text_colour",PROP_TYPE_COLOUR,offsetof(Constraint, text_color)},
+  { "line_colour",PROP_TYPE_COLOUR,offsetof(Constraint, line_color)},
   { NULL, 0, 0 }
 };
 
@@ -167,14 +179,15 @@ constraint_distance_from(Constraint *constraint, Point *point)
 
 static void
 constraint_select(Constraint *constraint, Point *clicked_point,
-	    Renderer *interactive_renderer)
+	    DiaRenderer *interactive_renderer)
 {
   connection_update_handles(&constraint->connection);
 }
 
-static void
+static ObjectChange*
 constraint_move_handle(Constraint *constraint, Handle *handle,
-		 Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		       Point *to, ConnectionPoint *cp,
+		       HandleMoveReason reason, ModifierKeys modifiers)
 {
   Point p1, p2;
   Point *endpoints;
@@ -189,7 +202,8 @@ constraint_move_handle(Constraint *constraint, Handle *handle,
     endpoints = &constraint->connection.endpoints[0]; 
     p1.x = 0.5*(endpoints[0].x + endpoints[1].x);
     p1.y = 0.5*(endpoints[0].y + endpoints[1].y);
-    connection_move_handle(&constraint->connection, handle->id, to, reason);
+    connection_move_handle(&constraint->connection, handle->id, to, cp, 
+			   reason, modifiers);
     p2.x = 0.5*(endpoints[0].x + endpoints[1].x);
     p2.y = 0.5*(endpoints[0].y + endpoints[1].y);
     point_sub(&p2, &p1);
@@ -197,9 +211,11 @@ constraint_move_handle(Constraint *constraint, Handle *handle,
   }
 
   constraint_update_data(constraint);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 constraint_move(Constraint *constraint, Point *to)
 {
   Point start_to_end;
@@ -218,41 +234,46 @@ constraint_move(Constraint *constraint, Point *to)
   point_add(&constraint->text_pos, &delta);
   
   constraint_update_data(constraint);
+
+  return NULL;
 }
 
 static void
-constraint_draw(Constraint *constraint, Renderer *renderer)
+constraint_draw(Constraint *constraint, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Point *endpoints;
-  
+  Arrow arrow;
+
   assert(constraint != NULL);
   assert(renderer != NULL);
 
   endpoints = &constraint->connection.endpoints[0];
   
-  renderer->ops->set_linewidth(renderer, CONSTRAINT_WIDTH);
-  renderer->ops->set_dashlength(renderer, CONSTRAINT_DASHLEN);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_DASHED);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  renderer_ops->set_linewidth(renderer, CONSTRAINT_WIDTH);
+  renderer_ops->set_dashlength(renderer, CONSTRAINT_DASHLEN);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_DASHED);
+  renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
 
-  renderer->ops->draw_line(renderer,
-			   &endpoints[0], &endpoints[1],
-			   &color_black);
+  arrow.type = ARROW_LINES;
+  arrow.length = CONSTRAINT_ARROWLEN;
+  arrow.width = CONSTRAINT_ARROWWIDTH;
+
+  renderer_ops->draw_line_with_arrows(renderer,
+				       &endpoints[0], &endpoints[1],
+				       CONSTRAINT_WIDTH,
+				       &constraint->line_color,
+				       NULL, &arrow);
   
-  arrow_draw(renderer, ARROW_LINES,
-	     &endpoints[1], &endpoints[0],
-	     CONSTRAINT_ARROWLEN, CONSTRAINT_ARROWWIDTH, CONSTRAINT_WIDTH,
-	     &color_black, &color_white);
-
-  renderer->ops->set_font(renderer, constraint_font,
+  renderer_ops->set_font(renderer, constraint_font,
 			  CONSTRAINT_FONTHEIGHT);
-  renderer->ops->draw_string(renderer,
+  renderer_ops->draw_string(renderer,
 			     constraint->brtext,
 			     &constraint->text_pos, ALIGN_LEFT,
-			     &color_black);
+			     &constraint->text_color);
 }
 
-static Object *
+static DiaObject *
 constraint_create(Point *startpoint,
 		  void *user_data,
 		  Handle **handle1,
@@ -260,13 +281,12 @@ constraint_create(Point *startpoint,
 {
   Constraint *constraint;
   Connection *conn;
-  Object *obj;
+  DiaObject *obj;
   Point defaultlen = { 1.0, 1.0 };
 
   if (constraint_font == NULL) {
-	  /* choose default font name for your locale. see also font_data structure
-	     in lib/font.c. if "Courier" works for you, it would be better.  */
-	  constraint_font = font_getfont(_("Courier"));
+    constraint_font = 
+      dia_font_new_from_style (DIA_FONT_MONOSPACE, CONSTRAINT_FONTHEIGHT);
   }
   
   constraint = g_malloc0(sizeof(Constraint));
@@ -283,6 +303,8 @@ constraint_create(Point *startpoint,
   
   connection_init(conn, 3, 0);
 
+  constraint->text_color = color_black;
+  constraint->line_color = attributes_get_foreground();
   constraint->text = g_strdup("");
   constraint->text_pos.x = 0.5*(conn->endpoints[0].x + conn->endpoints[1].x);
   constraint->text_pos.y = 0.5*(conn->endpoints[0].y + conn->endpoints[1].y);
@@ -314,7 +336,7 @@ static void
 constraint_update_data(Constraint *constraint)
 {
   Connection *conn = &constraint->connection;
-  Object *obj = &conn->object;
+  DiaObject *obj = &conn->object;
   Rectangle rect;
   LineBBExtras *extra;
 
@@ -329,10 +351,9 @@ constraint_update_data(Constraint *constraint)
   
   obj->position = conn->endpoints[0];
 
-  constraint->text_width =
-    font_string_width(constraint->brtext, 
-                      constraint_font, 
-                      CONSTRAINT_FONTHEIGHT);
+  constraint->text_width = dia_font_string_width(constraint->brtext, 
+                                                 constraint_font, 
+                                                 CONSTRAINT_FONTHEIGHT);
   
   constraint->text_handle.pos = constraint->text_pos;
 
@@ -351,13 +372,15 @@ constraint_update_data(Constraint *constraint)
   /* Add boundingbox for text: */
   rect.left = constraint->text_pos.x;
   rect.right = rect.left + constraint->text_width;
-  rect.top = constraint->text_pos.y - font_ascent(constraint_font, CONSTRAINT_FONTHEIGHT);
+  rect.top = constraint->text_pos.y -
+      dia_font_ascent(constraint->brtext,
+                      constraint_font, CONSTRAINT_FONTHEIGHT);
   rect.bottom = rect.top + CONSTRAINT_FONTHEIGHT;
   rectangle_union(&obj->bounding_box, &rect);
 }
 
 
-static Object *
+static DiaObject *
 constraint_load(ObjectNode obj_node, int version, const char *filename)
 {
   return object_load_using_properties(&constraint_type,
